@@ -1,11 +1,12 @@
 """State module for managing Disks."""
-import copy
 from dataclasses import field
 from dataclasses import make_dataclass
 from typing import Any
 from typing import Dict
 from typing import List
 
+from dict_tools.typing import Computed
+
 __contracts__ = ["resource"]
 
 
@@ -15,6 +16,7 @@ async def present(
     name: str = None,
     project: str = None,
     zone: str = None,
+    region: str = None,
     resource_id: str = None,
     request_id: str = None,
     source_image: str = None,
@@ -22,10 +24,10 @@ async def present(
     source_snapshot: str = None,
     source_storage_object: str = None,
     options: str = None,
-    type: str = None,
+    type_: str = None,
     licenses: List[str] = None,
     guest_os_features: List[
-        make_dataclass("GuestOsFeature", [("type", str, field(default=None))])
+        make_dataclass("GuestOsFeature", [("type_", str, field(default=None))])
     ] = None,
     disk_encryption_key: make_dataclass(
         "CustomerEncryptionKey",
@@ -71,6 +73,7 @@ async def present(
         ],
     ) = None,
     description: str = None,
+    id_: Computed[str] = None,
 ) -> Dict[str, Any]:
     r"""Creates or updates a persistent disk in the specified project using the data in the request.
 
@@ -81,9 +84,12 @@ async def present(
         name(str):
             An Idem name of the resource.
 
-        zone(str):
+        zone(str, Optional):
             The name of the zone for this request.
 
+        region(str, Optional):
+            The name of the region for this request.
+
         project(str, Optional):
             Project ID for this request.
 
@@ -144,7 +150,7 @@ async def present(
             Internal use only.
             Defaults to None.
 
-        type(str, Optional):
+        type_(str, Optional):
             URL of the disk type resource describing which disk type to use to create the disk.
             Provide this when creating the disk. For example: projects/project /zones/zone/diskTypes/pd-ssd.
             See Persistent disk types.
@@ -159,7 +165,7 @@ async def present(
             Read Enabling guest operating system features to see a list of available options.
             Defaults to None.
 
-                * type (str, Optional):
+                * type_ (str, Optional):
                     The ID of a supported feature. To add multiple values,
                     use commas to separate values. Set to one or more of the following values:
                         - VIRTIO_SCSI_MULTIQUEUE
@@ -269,6 +275,7 @@ async def present(
 
         resource_policies(List[str], Optional):
             Resource policies applied to this disk for automatic snapshot creations.
+            To remove a policy set resource_policies to be an empty array e.g. []
             Defaults to None.
 
         source_disk(str, Optional):
@@ -311,6 +318,8 @@ async def present(
             An optional description of this resource.
             Defaults to None.
 
+        id_(str, optional): The unique identifier for the resource. This identifier is defined by the server. Read-only property
+
     Returns:
         Dict[str, Any]
 
@@ -322,7 +331,7 @@ async def present(
                 - project: project-name
                 - zone: us-central1-a
                 - description: Description for example_resource_name
-                - type: projects/project-name/zones/us-central1-a/diskTypes/pd-balanced
+                - type_: projects/project-name/zones/us-central1-a/diskTypes/pd-balanced
                 - size_gb: 10
                 - resource_policies:
                     - projects/project-name/regions/us-central1/resourcePolicies/default-schedule-1
@@ -338,13 +347,11 @@ async def present(
     project = hub.tool.gcp.utils.get_project_from_account(ctx, project)
 
     if ctx.get("wrapper_result"):
-        result["old_state"] = copy.deepcopy(
-            copy.copy(ctx.get("wrapper_result").get("old_state", None))
-        )
+        result["old_state"] = ctx.get("wrapper_result").get("old_state", None)
 
     # to be autogenerated by pop-create based on insert/update props in properties.yaml
     resource_body = {
-        "type": type,
+        "type_": type_,
         "name": name,
         "source_image": source_image,
         "label_fingerprint": label_fingerprint,
@@ -359,6 +366,7 @@ async def present(
         "license_codes": license_codes,
         "source_snapshot_encryption_key": source_snapshot_encryption_key,
         "zone": zone,
+        "region": region,
         "source_image_encryption_key": source_image_encryption_key,
         "physical_block_size_bytes": physical_block_size_bytes,
         "source_snapshot": source_snapshot,
@@ -369,6 +377,7 @@ async def present(
         "location_hint": location_hint,
         "disk_encryption_key": disk_encryption_key,
         "provisioned_iops": provisioned_iops,
+        "id_": id_,
     }
 
     # TODO: How to handle query params which are not returned in the response body of get
@@ -401,9 +410,12 @@ async def present(
 
                 if hub.tool.gcp.operation_utils.is_operation(update_ret["ret"]):
                     operation = update_ret["ret"]
+                    operation_type = hub.tool.gcp.operation_utils.get_operation_type(
+                        operation.get("selfLink")
+                    )
                     operation_id = (
                         hub.tool.gcp.resource_prop_utils.parse_link_to_resource_id(
-                            operation.get("selfLink"), "compute.zone_operation"
+                            operation.get("selfLink"), operation_type
                         )
                     )
                     handle_operation_ret = (
@@ -429,13 +441,20 @@ async def present(
                     "labels": labels,
                     "label_fingerprint": label_fingerprint,
                 }
+                kwargs = {
+                    "project": project,
+                    "resource": name,
+                    "body": set_labels_body,
+                    "request_id": request_id,
+                }
+                if zone:
+                    kwargs.update({"zone": zone})
+                if region:
+                    kwargs.update({"region": region})
+
                 update_ret = await hub.exec.gcp_api.client.compute.disk.setLabels(
                     ctx,
-                    project=project,
-                    zone=zone,
-                    resource=name,
-                    body=set_labels_body,
-                    request_id=request_id,
+                    **kwargs,
                 )
 
                 if not update_ret["result"]:
@@ -445,9 +464,12 @@ async def present(
 
                 if hub.tool.gcp.operation_utils.is_operation(update_ret["ret"]):
                     operation = update_ret["ret"]
+                    operation_type = hub.tool.gcp.operation_utils.get_operation_type(
+                        operation.get("selfLink")
+                    )
                     operation_id = (
                         hub.tool.gcp.resource_prop_utils.parse_link_to_resource_id(
-                            operation.get("selfLink"), "compute.zone_operation"
+                            operation.get("selfLink"), operation_type
                         )
                     )
                     handle_operation_ret = (
@@ -525,8 +547,13 @@ async def present(
 
                         if hub.tool.gcp.operation_utils.is_operation(remove_ret["ret"]):
                             operation = remove_ret["ret"]
+                            operation_type = (
+                                hub.tool.gcp.operation_utils.get_operation_type(
+                                    operation.get("selfLink")
+                                )
+                            )
                             operation_id = hub.tool.gcp.resource_prop_utils.parse_link_to_resource_id(
-                                operation.get("selfLink"), "compute.zone_operation"
+                                operation.get("selfLink"), operation_type
                             )
                             handle_operation_ret = (
                                 await hub.tool.gcp.operation_utils.handle_operation(
@@ -557,8 +584,13 @@ async def present(
 
                         if hub.tool.gcp.operation_utils.is_operation(add_ret["ret"]):
                             operation = add_ret["ret"]
+                            operation_type = (
+                                hub.tool.gcp.operation_utils.get_operation_type(
+                                    operation.get("selfLink")
+                                )
+                            )
                             operation_id = hub.tool.gcp.resource_prop_utils.parse_link_to_resource_id(
-                                operation.get("selfLink"), "compute.zone_operation"
+                                operation.get("selfLink"), operation_type
                             )
                             handle_operation_ret = (
                                 await hub.tool.gcp.operation_utils.handle_operation(
@@ -621,9 +653,13 @@ async def present(
             return result
 
         # Create
-        create_ret = await hub.exec.gcp_api.client.compute.disk.insert(
-            ctx, name=name, project=project, zone=zone, body=resource_body
-        )
+        kwargs = {"name": name, "project": project, "body": resource_body}
+        if zone:
+            kwargs.update({"zone": zone})
+        if region:
+            kwargs.update({"region": region})
+
+        create_ret = await hub.exec.gcp_api.client.compute.disk.insert(ctx, **kwargs)
         if not create_ret["result"]:
             result["result"] = False
             result["comment"] += create_ret["comment"]
@@ -637,8 +673,11 @@ async def present(
                 operation = create_ret["ret"]
 
     if operation:
+        operation_type = hub.tool.gcp.operation_utils.get_operation_type(
+            operation.get("selfLink")
+        )
         operation_id = hub.tool.gcp.resource_prop_utils.parse_link_to_resource_id(
-            operation.get("selfLink"), "compute.zone_operation"
+            operation.get("selfLink"), operation_type
         )
         result["rerun_data"] = {
             "operation_id": operation_id,
@@ -655,6 +694,7 @@ async def absent(
     name: str = None,
     project: str = None,
     resource_id: str = None,
+    zone: str = None,
     request_id: str = None,
 ) -> Dict[str, Any]:
     r"""Deletes the resource.
@@ -672,6 +712,9 @@ async def absent(
         resource_id(str, Optional):
             An identifier of the resource in the provider. Defaults to None.
 
+        zone(str, Optional):
+            The name of the zone for this request.
+
     Returns:
         Dict[str, Any]
     """
