@@ -4,7 +4,6 @@ Copyright (c) 2023 VMware, Inc. All Rights Reserved.
 SPDX-License-Identifier: Apache-2.0
 
 """
-import copy
 from dataclasses import field
 from dataclasses import make_dataclass
 from typing import Any
@@ -229,7 +228,7 @@ async def present(
                 )
             return result
 
-        result["old_state"] = copy.deepcopy(copy.copy(old_get_ret["ret"]))
+        result["old_state"] = old_get_ret["ret"]
     elif not get_resource_only_with_resource_id:
         resource_id = hub.tool.gcp.resource_prop_utils.construct_resource_id(
             "compute.firewall", {"project": project, "firewall": name}
@@ -244,7 +243,7 @@ async def present(
             return result
 
         if old_get_ret["ret"]:
-            result["old_state"] = copy.deepcopy(copy.copy(old_get_ret["ret"]))
+            result["old_state"] = old_get_ret["ret"]
 
     resource_body = {
         "priority": priority,
