@@ -9,8 +9,8 @@ __contracts__ = ["resource"]
 
 from dataclasses import field
 from dataclasses import make_dataclass
+from dict_tools.typing import Computed
 from typing import Dict, Any, List
-import copy
 
 from idem_gcp.tool.gcp.state_operation_utils import StateOperations
 
@@ -409,6 +409,7 @@ async def present(
     minimal_action: str = None,
     most_disruptive_allowed_action: str = None,
     resource_id: str = None,
+    id_: Computed[str] = None,
 ) -> Dict[str, Any]:
     r"""Create or update a compute instance resource.
 
@@ -1067,13 +1068,15 @@ async def present(
         resource_id(str, Optional):
             An identifier of the resource in the provider. Defaults to None.
 
+        id_(str, optional): The unique identifier for the resource. This identifier is defined by the server. Read-only property
+
     Returns:
         Dict[str, Any]
 
     Examples:
         .. code-block:: sls
 
-        example_resource_name:
+        example-resource-name:
           gcp.compute.instance.present:
           - project: example-project
           - machine_type: https://www.googleapis.com/compute/v1/projects/example-project/zones/us-central1-a/machineTypes/g1-small
@@ -1085,7 +1088,7 @@ async def present(
                 name: External NAT
                 network_tier: PREMIUM
                 set_public_ptr: false
-                type: ONE_TO_ONE_NAT
+                type_: ONE_TO_ONE_NAT
               kind: compute#networkInterface
               name: nic0
               network: https://www.googleapis.com/compute/v1/projects/project-name/global/networks/default
@@ -1097,7 +1100,7 @@ async def present(
               device_name: example_disk_1
               source: https://www.googleapis.com/compute/v1/projects/project-name/zones/us-central1-a/disks/example_disk_1
               mode: READ_WRITE
-              type: PERSISTENT
+              type_: PERSISTENT
               disk_size_gb: '10'
               index: 0
               interface: SCSI
@@ -1128,9 +1131,7 @@ async def present(
     project = hub.tool.gcp.utils.get_project_from_account(ctx, project)
 
     if ctx.get("wrapper_result"):
-        result["old_state"] = copy.deepcopy(
-            copy.copy(ctx.get("wrapper_result").get("old_state", None))
-        )
+        result["old_state"] = ctx.get("wrapper_result").get("old_state", None)
 
     # to be autogenerated by pop-create based on insert/update props in properties.yaml
     resource_body = {
@@ -1164,15 +1165,13 @@ async def present(
         "display_device": display_device,
         "zone": zone,
         "status": status,
+        "id_": id_,
     }
 
     resource_body = {k: v for (k, v) in resource_body.items() if v is not None}
     operation = None
     if result["old_state"]:
         resource_id = result["old_state"].get("resource_id", None)
-        # copy.copy(old_get_ret['ret']) is needed to convert any objects of type NamespaceDict to dict
-        # in old_get_ret['ret']. This is done so that comparisons with the resource_body which has
-        # objects of type dict works properly
         zone = hub.tool.gcp.resource_prop_utils.parse_link_to_zone(
             result["old_state"]["zone"]
         )
@@ -1229,9 +1228,9 @@ async def present(
             {
                 "resource_id": resource_id,
                 **resource_body,
-                **state_operations.old_properties_dict,
             },
             "compute.instance",
+            additional_exclude_paths=list(patch_operations_dict.keys()),
         )
 
         if changes:
@@ -1249,7 +1248,7 @@ async def present(
                         changed_non_updatable_properties,
                     )
                 )
-                result["new_state"] = copy.deepcopy(result["old_state"])
+                result["new_state"] = result["old_state"]
                 return result
 
         if not changes and not any(state_operations.changed_properties_dict.values()):
@@ -1259,7 +1258,7 @@ async def present(
                     "gcp.compute.instance", name
                 )
             )
-            result["new_state"] = copy.deepcopy(result["old_state"])
+            result["new_state"] = result["old_state"]
             return result
 
         if ctx["test"]:
