@@ -1,5 +1,4 @@
 """State module for managing Machine Images."""
-import copy
 from dataclasses import field
 from dataclasses import make_dataclass
 from typing import Any
@@ -135,9 +134,7 @@ async def present(
     project = hub.tool.gcp.utils.get_project_from_account(ctx, project)
 
     if ctx.get("wrapper_result"):
-        result["old_state"] = copy.deepcopy(
-            copy.copy(ctx.get("wrapper_result").get("old_state", None))
-        )
+        result["old_state"] = ctx.get("wrapper_result").get("old_state", None)
 
     request_body = {
         "storage_locations": storage_locations,
@@ -168,7 +165,7 @@ async def present(
                     "gcp.compute.machine_image", name
                 )
             )
-            result["new_state"] = copy.deepcopy(result["old_state"])
+            result["new_state"] = result["old_state"]
             return result
 
         if ctx["test"]:
