@@ -1,11 +1,12 @@
 """State module for managing Networks."""
-import copy
 from dataclasses import field
 from dataclasses import make_dataclass
 from typing import Any
 from typing import Dict
 from typing import List
 
+from dict_tools.typing import Computed
+
 __contracts__ = ["resource"]
 
 
@@ -48,6 +49,7 @@ async def present(
             ],
         )
     ] = None,
+    id_: Computed[str] = None,
 ) -> Dict[str, Any]:
     r"""Creates a network in the specified project using the data included in the request.
 
@@ -120,6 +122,8 @@ async def present(
                 "IPV4_IPV6" - This Peering will allow IPv4 traffic and routes to be exchanged. Additionally if the matching peering is IPV4_IPV6, IPv6 traffic and routes will be exchanged as well.
                 "IPV4_ONLY" - This Peering will only allow IPv4 traffic and routes to be exchanged, even if the matching peering is IPV4_IPV6.
 
+        id_(str, optional): The unique identifier for the resource. This identifier is defined by the server. Read-only property
+
     Returns:
         Dict[str, Any]
 
@@ -169,7 +173,7 @@ async def present(
             result["comment"] += old_get_ret["comment"]
             return result
 
-        result["old_state"] = copy.deepcopy(copy.copy(old_get_ret["ret"]))
+        result["old_state"] = old_get_ret["ret"]
     elif not hub.OPT.idem.get("get_resource_only_with_resource_id", False):
         resource_id = (ctx.get("old_state") or {}).get(
             "resource_id"
@@ -185,7 +189,7 @@ async def present(
             return result
 
         if old_get_ret["ret"]:
-            result["old_state"] = copy.deepcopy(copy.copy(old_get_ret["ret"]))
+            result["old_state"] = old_get_ret["ret"]
 
     request_body = {
         "name": name,
@@ -198,6 +202,7 @@ async def present(
         "network_firewall_policy_enforcement_order": network_firewall_policy_enforcement_order,
         "routing_config": routing_config,
         "peerings": peerings,
+        "id_": id_,
     }
 
     request_body = {k: v for (k, v) in request_body.items() if v is not None}
@@ -207,7 +212,7 @@ async def present(
             "resource_id": resource_id,
             **request_body,
         }
-        if hasattr(result["old_state"], "peerings"):
+        if result["old_state"].get("peerings"):
             new_state_for_whole_resource_comparison["peerings"] = result[
                 "old_state"
             ].get("peerings")
