@@ -1,11 +1,12 @@
 """State module for managing subnetworks."""
-import copy
 from dataclasses import field
 from dataclasses import make_dataclass
 from typing import Any
 from typing import Dict
 from typing import List
 
+from dict_tools.typing import Computed
+
 from idem_gcp.tool.gcp.state_operation_utils import StateOperations
 
 __contracts__ = ["resource"]
@@ -51,6 +52,7 @@ async def present(
     ip_cidr_range: str = None,
     drain_timeout_seconds: int = None,
     resource_id: str = None,
+    id_: Computed[str] = None,
 ) -> Dict[str, Any]:
     r"""Creates a subnetwork in the specified project using the data included in the request.
 
@@ -151,6 +153,8 @@ async def present(
         resource_id(str, Optional):
             An identifier of the resource in the provider. Defaults to None.
 
+        id_(str, optional): The unique identifier for the resource. This identifier is defined by the server. Read-only property
+
     Returns:
         Dict[str, Any]
 
@@ -174,9 +178,7 @@ async def present(
     project = hub.tool.gcp.utils.get_project_from_account(ctx, project)
 
     if ctx.get("wrapper_result"):
-        result["old_state"] = copy.deepcopy(
-            copy.copy(ctx.get("wrapper_result").get("old_state", None))
-        )
+        result["old_state"] = ctx.get("wrapper_result").get("old_state", None)
     request_body = {
         "log_config": log_config,
         "purpose": purpose,
@@ -191,6 +193,7 @@ async def present(
         "stack_type": stack_type,
         "secondary_ip_ranges": secondary_ip_ranges,
         "network": network,
+        "id_": id_,
     }
 
     operation = None
@@ -213,13 +216,13 @@ async def present(
         )
 
         changes = hub.tool.gcp.utils.compare_states(
-            copy.copy(result["old_state"]),
+            result["old_state"],
             {
                 "resource_id": resource_id,
                 **request_body,
-                **state_operations.old_properties_dict,
             },
             "compute.subnetwork",
+            additional_exclude_paths=list(patch_operations_dict.keys()),
         )
 
         if not changes and not any(state_operations.changed_properties_dict.values()):
@@ -229,7 +232,7 @@ async def present(
                     "gcp.compute.subnetwork", name
                 )
             )
-            result["new_state"] = copy.deepcopy(result["old_state"])
+            result["new_state"] = result["old_state"]
             return result
 
         if ctx["test"]:
