@@ -5,7 +5,11 @@ from idem_gcp.tool.gcp.generate.exec_context import ExecutionContext
 
 
 def __init__(hub):
-    hub.tool.gcp.generate.PRE_EXECS_ORDER = ["resource_id_extractor", "function_getter"]
+    hub.tool.gcp.generate.PRE_EXECS_ORDER = [
+        "resource_id_extractor",
+        "parameter_sanitizer",
+        "function_getter",
+    ]
     hub.tool.gcp.generate.POST_EXECS_ORDER = ["operation_processor", "item_getter"]
 
 
