@@ -18,6 +18,14 @@ async def operate(hub, execution_context: ExecutionContext) -> None:
                 execution_context.resource_type, resource_id
             )
         )
-        extended_params.update(resource_id_params)
+        aliases = execution_context.exec_params[ExecParam.PATH_PARAM_ALIASES]
+        if not aliases:
+            extended_params.update(resource_id_params)
+        else:
+            for resource_id_param in resource_id_params:
+                if resource_id_param in aliases:
+                    extended_params[aliases[resource_id_param]] = resource_id_params[
+                        resource_id_param
+                    ]
 
     execution_context.method_params = extended_params
