@@ -17,9 +17,11 @@ them to the appropriate GCP Python SDK wrappers within the tool Sub of this
 project.
 """
 import copy
+import uuid
 from inspect import signature
 from typing import Dict
 from typing import List
+from typing import Optional
 
 from pop.loader import LoadedMod
 
@@ -39,22 +41,30 @@ def resolve_sub(hub, ref: str, gcp_api):
 
     def gen_sub(ref: str, gcp_api):
         callpath = ref.split(".")
+        method_name = callpath[-1]
         resource_type = ".".join(callpath[1:-1])
-        native_resource_type = hub.tool.gcp.resolver.find_native_resource_type(
+        native_resource_types = hub.tool.gcp.resolver.find_native_resource_types(
             resource_type
         )
-        method_name = callpath[-1]
-        callpath = callpath[0:1] + native_resource_type.split(".") + [callpath[-1]]
-        callpath[0:-1] = [hub.tool.gcp.case.camel(v) for v in callpath[0:-1]]
+        assert len(native_resource_types) > 0
+        cms = []
+        for nr_type in native_resource_types:
+            cpath = callpath[0:1] + nr_type.split(".") + [method_name]
+            cpath[0:-1] = [hub.tool.gcp.case.camel(v) for v in cpath[0:-1]]
 
-        gcp_api_sub = gcp_api
-        for c in callpath[1:]:
-            gcp_api_sub = getattr(gcp_api_sub, c)
-        cm = ContractMod(
-            hub, gcp_api_sub, resource_type, native_resource_type, method_name
+            gcp_api_sub = gcp_api
+            for c in cpath[1:]:
+                gcp_api_sub = getattr(gcp_api_sub, c)
+            cm = ContractMod(hub, gcp_api_sub, resource_type, nr_type, method_name)
+            cms.append(cm)
+
+        _GCP_SUB_CACHE[ref] = (
+            CompositeContractMod(hub, resource_type, method_name, cms)
+            if len(cms) > 1
+            else cms[0]
         )
-        _GCP_SUB_CACHE[ref] = cm
-        return cm
+
+        return _GCP_SUB_CACHE[ref]
 
     return _GCP_SUB_CACHE.get(ref) or gen_sub(ref, gcp_api)
 
@@ -155,6 +165,11 @@ class ContractMod(LoadedMod):
         ):
             raw_params["projection"] = "full"
 
+        if is_request_property(
+            self._hub, "name", self._resource_type, self._method_name
+        ):
+            raw_params["name"] = kwargs["name"]
+
         try:
             response = await self._gcp_method(ctx, *args, **raw_params)
             # Is it possible to get None for response. It looks like on error exceptions is thrown. If we are here
@@ -170,6 +185,12 @@ class ContractMod(LoadedMod):
                 result["ret"] = response
                 return result
 
+            response_field = get_response_field(
+                self._hub, self._resource_type, self._method_name
+            )
+            if response_field:
+                response["items"] = response[response_field]
+
             items = response.get("items")
             if isinstance(items, dict):
                 response[
@@ -229,6 +250,110 @@ class ContractMod(LoadedMod):
         return result
 
 
+class CompositeContractMod(LoadedMod):
+    def __init__(
+        self, hub, resource_type: str, method_name: str, mods: List[ContractMod]
+    ):
+        super().__init__(name=f"composite:{uuid.uuid4()}")
+        self._hub = hub
+        self._resource_type = resource_type
+        self._method_name = method_name
+        self._mods = mods
+
+    @property
+    def signature(self):
+        return signature(self.__call__)
+
+    def _missing(self, item: str):
+        return self
+
+    async def __call__(self, ctx, *args, **kwargs):
+        result = {"result": False, "ret": None, "comment": []}
+
+        pending_comments = []
+        mods_to_call = []
+        if self.should_aggregate_results():
+            mods_to_call.extend(self._mods)
+        else:
+            mod = self.find_best_matching_mod(**kwargs)
+            if mod:
+                mods_to_call.append(mod)
+
+        for mod in mods_to_call:
+            result_from_mod = await mod(ctx, *args, **kwargs)
+            if result_from_mod["result"]:
+                result["comment"] += result_from_mod["comment"]
+                result["result"] = True
+                if result["ret"]:
+                    result["ret"].extend(result_from_mod["ret"])
+                else:
+                    result["ret"] = result_from_mod["ret"]
+            else:
+                pending_comments += result_from_mod["comment"]
+
+        if not result["result"]:
+            result["comment"].extend(pending_comments)
+
+        return result
+
+    def find_best_matching_mod(self, **kwargs) -> Optional[ContractMod]:
+        resource_paths = self._hub.tool.gcp.resource_prop_utils.get_resource_paths(
+            self._resource_type
+        )
+        assert len(self._mods) == len(resource_paths)
+
+        result = None
+        multiple_max_scores = False
+        max_score_so_far = -1
+        for idx, path in enumerate(resource_paths):
+            score = self.compute_matching_score(path, **kwargs)
+            if score > max_score_so_far:
+                max_score_so_far = score
+                result = self._mods[idx]
+                multiple_max_scores = False
+            elif score == max_score_so_far:
+                multiple_max_scores = True
+
+        if multiple_max_scores:
+            msg = f"Multiple resource paths of {self._resource_type} match the input arguments."
+            self._hub.log.error(msg)
+            return None
+
+        if not result:
+            msg = (
+                f"No resource paths of {self._resource_type} match the input arguments."
+            )
+            self._hub.log.warning(msg)
+            return None
+
+        return result
+
+    def compute_matching_score(self, resource_path: str, **kwargs) -> int:
+        score = 0
+        params = self._hub.tool.gcp.resource_prop_utils.get_path_parameters_for_path(
+            resource_path
+        )
+
+        if "resource_id" in kwargs:
+            resource_id = kwargs.get("resource_id")
+            if (
+                self._hub.tool.gcp.resource_prop_utils.parse_link_to_resource_id_and_path(
+                    resource_id, self._resource_type, resource_path
+                )
+                is not None
+            ):
+                score = len(params)
+        else:
+            for param in params:
+                if param in kwargs:
+                    score += 1
+
+        return score
+
+    def should_aggregate_results(self) -> bool:
+        return self._method_name in ["list", "aggregatedList"]
+
+
 def process_response(hub, response: Dict, resource_type: str, method_name: str) -> Dict:
     # TODO: Get does not return all the necessary properties which should be the present properties so
     #  decide what to do
@@ -277,8 +402,6 @@ def process_response(hub, response: Dict, resource_type: str, method_name: str)
 
         return {"items": ret_items}
 
-    return {}
-
 
 def filter_raw_properties_to_present(
     hub,
@@ -293,9 +416,7 @@ def filter_raw_properties_to_present(
         if key in present_properties_names
     }
     filtered["resource_id"] = resource_id
-    filtered = hub.tool.gcp.resource_prop_utils.format_path_params(
-        filtered, resource_type
-    )
+    hub.tool.gcp.resource_prop_utils.format_path_params(filtered, resource_type)
     return hub.tool.gcp.conversion_utils.convert_raw_resource_to_present(
         filtered, resource_type
     )
@@ -331,7 +452,7 @@ def has_singular_result(
     return True
 
 
-def find_native_resource_type(hub, resource_type: str) -> str:
+def find_native_resource_types(hub, resource_type: str) -> str:
     def fetch_res_type_mapping(res_type: str) -> str:
         hub_ref = hub.metadata.gcp
         for part in res_type.split("."):
@@ -342,9 +463,37 @@ def find_native_resource_type(hub, resource_type: str) -> str:
             hub.log.warning(f"Can't find native resource type for {resource_type}")
             ret = res_type
 
-        _GCP_RESOURCE_TYPES_CACHE[res_type] = ret
-        return ret
+        _GCP_RESOURCE_TYPES_CACHE[res_type] = ret if isinstance(ret, list) else [ret]
+        return _GCP_RESOURCE_TYPES_CACHE[res_type]
 
     return _GCP_RESOURCE_TYPES_CACHE.get(resource_type) or fetch_res_type_mapping(
         resource_type
     )
+
+
+def is_request_property(hub, prop: str, resource_type: str, method_name: str) -> bool:
+    """
+    Returns true if the specified property is a path parameter.
+    """
+    resource_methods_properties = hub.tool.gcp.RESOURCE_PROPS[resource_type]
+    requests_parameters = resource_methods_properties.get("request_parameters", {})
+    method_params = requests_parameters.get(method_name, [])
+    return prop in method_params
+
+
+def get_response_field(hub, resource_type: str, method_name: str) -> str:
+    """
+    Returns the name of the property which contains the actual resource inside the response body.
+
+    For example: the response body when listing service accounts has the following structure:
+    {
+      "accounts": [
+        object (ServiceAccount)
+      ],
+      "nextPageToken": string
+    }
+    In this case the result will be 'accounts'.
+    """
+    resource_methods_properties = hub.tool.gcp.RESOURCE_PROPS[resource_type]
+    response_fields = resource_methods_properties.get("response_fields", {})
+    return response_fields.get(method_name)
