@@ -1,4 +1,3 @@
-import copy
 import re
 from typing import Any
 from typing import Dict
@@ -221,17 +220,42 @@ def extract_resource_id(hub, input_props: Dict, resource_type: str) -> Dict:
                 return input_props[resource_id_field]
 
 
-def get_resource_path(hub, resource_type: str) -> str:
+def get_resource_paths(hub, resource_type: str) -> List[str]:
     resource_path = resource_type.split(".")
     hub_ref = hub.metadata.gcp
     for resource_path_segment in resource_path:
         hub_ref = hub_ref[resource_path_segment]
 
-    return hub_ref["PATH"]
+    result = hub_ref["PATH"]
+    return result if isinstance(result, list) else [result]
 
 
 def get_elements_from_resource_id(hub, resource_type: str, resource_id: str) -> Dict:
-    resource_path = hub.tool.gcp.resource_prop_utils.get_resource_path(resource_type)
+    resource_paths = hub.tool.gcp.resource_prop_utils.get_resource_paths(resource_type)
+
+    result = None
+    for path in resource_paths:
+        r = hub.tool.gcp.resource_prop_utils.get_elements_from_resource_id_and_path(
+            resource_type, resource_id, path
+        )
+        if r:
+            if result:
+                msg = f"Resource ID {resource_id} matches multiple resource paths of {resource_type}"
+                hub.log.error(msg)
+                return {}
+            result = r
+
+    if not result:
+        msg = f"Resource ID {resource_id} matches no resource paths of {resource_type}"
+        hub.log.warning(msg)
+        return {}
+
+    return result
+
+
+def get_elements_from_resource_id_and_path(
+    hub, resource_type: str, resource_id: str, resource_path: str
+) -> Dict:
     src = resource_path.split("/")
     act = resource_id.split("/")
     result = {}
@@ -262,9 +286,10 @@ def get_elements_from_resource_id(hub, resource_type: str, resource_id: str) ->
     return result
 
 
-def parse_link_to_resource_id(hub, link, resource_type: str):
-    resource_path = hub.tool.gcp.resource_prop_utils.get_resource_path(resource_type)
-    result = copy.deepcopy(resource_path)
+def parse_link_to_resource_id_and_path(
+    hub, link, resource_type: str, resource_path: str
+) -> str:
+    result = resource_path
     src = result.split("/")
     act = link.split("/")
 
@@ -277,45 +302,86 @@ def parse_link_to_resource_id(hub, link, resource_type: str):
         if s.startswith("{") and s.endswith("}"):
             result = result.replace(s, val)
         elif val != s:
-            hub.log.warning(
-                f"Resource selfLink differs from the actual path for resource type: {resource_type}"
-                f"\nselfLink: {link}, PATH: {resource_path}"
-            )
             return None
         idx -= 1
 
     return result
 
 
+def parse_link_to_resource_id(hub, link, resource_type: str) -> str:
+    resource_paths = hub.tool.gcp.resource_prop_utils.get_resource_paths(resource_type)
+    result = None
+    for path in resource_paths:
+        r = hub.tool.gcp.resource_prop_utils.parse_link_to_resource_id_and_path(
+            link, resource_type, path
+        )
+        if r:
+            if result:
+                msg = f"Link {link} matches multiple resource paths of {resource_type}"
+                hub.log.error(msg)
+                return None
+            result = r
+
+    if not result:
+        msg = f"Link {link} matches no resource paths of {resource_type}"
+        hub.log.warning(msg)
+        return None
+
+    return result
+
+
 def construct_resource_id(hub, resource_type: str, input_props: Dict[str, Any]) -> str:
     if input_props is not None and input_props.get("name") is not None:
         # handle case where "name" contains full resource id
         name = input_props.get("name")
-        if (
-            hub.tool.gcp.resource_prop_utils.parse_link_to_resource_id(
-                name, resource_type
-            )
-            is not None
-        ):
-            return name
+        try:
+            if (
+                hub.tool.gcp.resource_prop_utils.parse_link_to_resource_id(
+                    name, resource_type
+                )
+                is not None
+            ):
+                return name
+        except ValueError:
+            ...
 
-    resource_path = hub.tool.gcp.resource_prop_utils.get_resource_path(resource_type)
-    try:
-        filtered_without_none_properties = {
-            k: v for k, v in input_props.items() if v is not None
-        }
-        return resource_path.format(**filtered_without_none_properties)
-    except:
-        # a required input prop is missing or input_props is invalid format
+    filtered_without_none_properties = (
+        {k: v for k, v in input_props.items() if v is not None}
+        if input_props is not None
+        else {}
+    )
+    resource_paths: List[str] = hub.tool.gcp.resource_prop_utils.get_resource_paths(
+        resource_type
+    )
+    result = None
+    for path in resource_paths:
+        try:
+            r = path.format(**filtered_without_none_properties)
+            if r.find("{") != -1 or r.find("}") != -1:
+                # not fully matched
+                r = None
+        except KeyError:
+            r = None
+        if r:
+            if result:
+                msg = f"Could not construct resource ID because multiple resource paths of {resource_type} match the input arguments."
+                hub.log.error(msg)
+                return None
+            result = r
+
+    if not result:
+        msg = f"Could not construct resource ID because no resource paths of {resource_type} match the input arguments."
+        hub.log.warning(msg)
         return None
 
+    return result
+
 
 def parse_link_to_zone(hub, link: str) -> str:
     return link.split("/")[-1]
 
 
-def get_path_parameters(hub, resource_type):
-    resource_path = hub.tool.gcp.resource_prop_utils.get_resource_path(resource_type)
+def get_path_parameters_for_path(hub, resource_path: str) -> Set[str]:
     src = resource_path.split("/")
     result = set()
 
@@ -326,14 +392,56 @@ def get_path_parameters(hub, resource_type):
     return result
 
 
-def format_path_params(hub, resource, resource_type):
-    resource_copy = copy.deepcopy(resource)
-    path_params = hub.tool.gcp.resource_prop_utils.get_path_parameters(resource_type)
-    for path_el in path_params:
-        if resource_copy.get(path_el):
-            resource_copy[path_el] = resource_copy[path_el].split("/")[-1]
+def get_path_parameters(hub, resource_type: str, resource: str) -> Set[str]:
+    resource_paths = hub.tool.gcp.resource_prop_utils.get_resource_paths(resource_type)
+    all_parameters = []
+    for path in resource_paths:
+        r = hub.tool.gcp.resource_prop_utils.get_path_parameters_for_path(path)
+        if r:
+            all_parameters.append(r)
 
-    return resource_copy
+    # Try to find best-matching path
+    result = None
+    for parameters in all_parameters:
+        missing = {p for p in parameters if p not in resource}
+        if not missing:
+            if result:
+                msg = f"Resource {resource} matches multiple resource paths of {resource_type}"
+                hub.log.error(msg)
+                return set()
+
+            result = parameters
+
+    if not result:
+        msg = f"Resource {resource} matches no resource paths of {resource_type}"
+        hub.log.warning(msg)
+        return set()
+
+    return result
+
+
+def get_path_parameters_combined(hub, resource_type: str, resource: str) -> Set[str]:
+    resource_paths = hub.tool.gcp.resource_prop_utils.get_resource_paths(resource_type)
+    all_parameters = []
+    for path in resource_paths:
+        r = hub.tool.gcp.resource_prop_utils.get_path_parameters_for_path(path)
+        if r:
+            all_parameters.append(r)
+
+    result = set()
+    for parameters in all_parameters:
+        result = result.union(parameters)
+
+    return result
+
+
+def format_path_params(hub, resource, resource_type):
+    path_params = hub.tool.gcp.resource_prop_utils.get_path_parameters_combined(
+        resource_type, resource
+    )
+    for path_el in path_params:
+        if resource.get(path_el):
+            resource[path_el] = resource[path_el].split("/")[-1]
 
 
 # TODO: Remove this logic once we have all the nested properties defined for a method
@@ -348,7 +456,23 @@ def are_properties_allowed_for_update(hub, resource_type, request_body):
 
 
 def resource_type_matches(hub, resource_id: str, resource_type: str) -> bool:
-    resource_path = hub.tool.gcp.resource_prop_utils.get_resource_path(resource_type)
+    resource_paths = hub.tool.gcp.resource_prop_utils.get_resource_paths(resource_type)
+    result = False
+    for path in resource_paths:
+        r = hub.tool.gcp.resource_prop_utils.resource_type_matches_for_path(
+            resource_id, path
+        )
+        if r:
+            if result:
+                msg = f"Resource ID {resource_id} matches multiple resource paths of {resource_type}"
+                hub.log.error(msg)
+                return False
+            result = True
+
+    return result
+
+
+def resource_type_matches_for_path(hub, resource_id: str, resource_path: str) -> bool:
     rpath = resource_path.split("/")
     rid = resource_id.split("/")
 
