@@ -8,8 +8,10 @@ import logging
 from copy import deepcopy
 from typing import Any
 from typing import Dict
+from typing import List
 
 from deepdiff import DeepDiff
+from dict_tools.data import NamespaceDict
 
 # Import plugin helpers
 
@@ -165,8 +167,7 @@ def merge_dicts(hub, target: dict, source: dict) -> dict:
     :param source: The dict from which to merge into the target.
     :return: dict as merged.
     """
-    result = copy.deepcopy(target)
-    return _merge_dicts(result, source)
+    return _merge_dicts(target, source)
 
 
 def is_pending(hub, ret: dict, state: str = None, **pending_kwargs) -> bool:
@@ -203,8 +204,13 @@ def compare_states(
     old_state: Dict,
     plan_state: Dict,
     resource_type: str = None,
+    additional_exclude_paths: List[str] = None,
 ) -> Dict:
-    exclude_paths = []
+    exclude_paths = (
+        hub.tool.gcp.utils.format_exclude_paths(additional_exclude_paths)
+        if additional_exclude_paths
+        else []
+    )
 
     for key in old_state.keys():
         if key not in plan_state:
@@ -215,7 +221,12 @@ def compare_states(
             hub.tool.gcp.utils.get_deep_diff_exclude_paths(resource_type)
         )
 
-    changes = DeepDiff(old_state, plan_state, exclude_regex_paths=exclude_paths)
+    changes = DeepDiff(
+        old_state,
+        plan_state,
+        exclude_regex_paths=exclude_paths,
+        ignore_type_in_groups=[(NamespaceDict, dict)],
+    )
 
     hub.tool.gcp.utils.filter_diff(changes, plan_state)
     # If an item is in old_state but not in plan_state, i.e. changes.get("dictionary_item_removed") is not None,
@@ -266,8 +277,12 @@ def get_plan_state_value_from_deep_diff_path(hub, deep_diff_path: str, plan_stat
 
 
 def get_deep_diff_exclude_paths(hub, resource_type: str):
-    exclude_paths = []
     paths = hub.tool.gcp.resource_prop_utils.get_exclude_paths(resource_type)
+    return hub.tool.gcp.utils.format_exclude_paths(paths)
+
+
+def format_exclude_paths(hub, paths):
+    exclude_paths = []
     for path in paths:
         s = "root"
         for part in path.split("."):
